@@ -14,7 +14,7 @@
 #  along with this program; if not, write to the Free Software
 #  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 
-# $Id: Apache2Handler.pm,v 1.7 2006/01/05 15:48:38 ssinyagin Exp $
+# $Id: Apache2Handler.pm,v 1.10 2007/03/16 16:07:41 ssinyagin Exp $
 # Stanislav Sinyagin <ssinyagin@yahoo.com>
 
 # Apache mod_perl handler. See http://perl.apache.org
@@ -143,6 +143,7 @@ sub handler : method
                 {
                     $session{'uid'} = $options{'uid'} = $uid;
                     $needsLogin = 0;
+                    Info('User logged in: ' . $uid);
                 }
                 else
                 {
@@ -283,7 +284,7 @@ sub handler : method
     
     if( not $Torrus::Renderer::globalDebug )
     {
-        &Torrus::Log::setLevel('warn');
+        &Torrus::Log::setLevel('info');
     }
     
     return $retval;
