@@ -14,7 +14,7 @@
 #  along with this program; if not, write to the Free Software
 #  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 
-# $Id: ApacheHandler.pm,v 1.7 2005/10/25 13:16:26 ssinyagin Exp $
+# $Id: ApacheHandler.pm,v 1.9 2007/03/16 16:07:41 ssinyagin Exp $
 # Stanislav Sinyagin <ssinyagin@yahoo.com>
 
 # Apache mod_perl handler. See http://perl.apache.org
@@ -136,6 +136,7 @@ sub handler
                 {
                     $session{'uid'} = $options{'uid'} = $args{'uid'};
                     $needsLogin = 0;
+                    Info('User logged in: ' . $args{'uid'});
                 }
                 else
                 {
@@ -290,7 +291,7 @@ sub handler
                             $Torrus::Global::cacheDir);            
     }
 
-    &Torrus::Log::setLevel('warn');
+    &Torrus::Log::setLevel('info');
     return $retval;
 }
 
