@@ -1,4 +1,4 @@
-#  Copyright (C) 2002  Stanislav Sinyagin
+#  Copyright (C) 2002-2007  Stanislav Sinyagin
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -14,7 +14,7 @@
 #  along with this program; if not, write to the Free Software
 #  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 
-# $Id: SNMP.pm,v 1.28 2007/01/25 08:37:46 ssinyagin Exp $
+# $Id: SNMP.pm,v 1.38 2007/07/27 15:56:00 ssinyagin Exp $
 # Stanislav Sinyagin <ssinyagin@yahoo.com>
 
 package Torrus::Collector::SNMP;
@@ -59,15 +59,42 @@ $Torrus::Collector::params{'snmp'} = {
 
 my $sysUpTime = '1.3.6.1.2.1.1.3.0';
 
+# Hosts that are running SNMPv1. We do not reresh maps on them, as
+# they are too slow
+my %snmpV1Hosts;
+
 # SNMP tables lookup maps
 my %maps;
 
+# How frequent we refresh the SNMP mapping
+our $mapsRefreshPeriod;
+
+# Random factor in refresh period
+our $mapsRefreshRandom;
+
+# how often we check for expired maps
+our $mapsExpireCheckPeriod;
+
+# expiration time for each map
+my %mapsExpire;
+
 # Lookups scheduled for execution
 my %mapLookupScheduled;
 
 # SNMP session objects for map lookups
 my @mappingSessions;
 
+
+# Timestamps of hosts last found unreachable
+my %hostUnreachableSeen;
+
+# Last time we tried to reach an unreachable host
+my %hostUnreachableRetry;
+
+# Hosts that were deleted because of unreachability for too long
+my %unreachableHostDeleted;
+
+
 # This is first executed per target
 
 $Torrus::Collector::initTarget{'snmp'} = \&Torrus::Collector::SNMP::initTarget;
@@ -88,12 +115,11 @@ sub initTarget
     my $ipaddr = getHostIpAddress( $collector, $token );
     if( not defined( $ipaddr ) )
     {
-        $collector->deleteTarget($token);
         return 0;
     }
 
     $tref->{'ipaddr'} = $ipaddr;
-
+    
     return Torrus::Collector::SNMP::initTargetAttributes( $collector, $token );
 }
 
@@ -125,6 +151,11 @@ sub initTargetAttributes
     my $hosthash = join('|', $ipaddr, $port, $community);
     $tref->{'hosthash'} = $hosthash;
 
+    if( $version eq '1' )
+    {
+        $snmpV1Hosts{$hosthash} = 1;
+    }
+    
     # If the object is defined as a map, retrieve the whole map
     # and cache it.
 
@@ -144,7 +175,7 @@ sub initTargetAttributes
 
     if( not $oid )
     {
-        if( $cref->{'unreachableHostDeleted'}{$hosthash} )
+        if( $unreachableHostDeleted{$hosthash} )
         {
             # we tried our best, but the target is dead
             return 0;
@@ -166,10 +197,7 @@ sub initTargetAttributes
     # There can be several targets with the same host|port|community+oid set.
 
     $cref->{'targets'}{$hosthash}{$oid}{$token} = 1;
-
-    # One representative for each host|port|community triple.
-    # I assume overridiing is faster than checking if it's already there
-    $cref->{'reptoken'}{$hosthash} = $token;
+    $cref->{'activehosts'}{$hosthash} = 1;
 
     $tref->{'oid'} = $oid;
 
@@ -243,42 +271,50 @@ sub snmpSessionArgs
     my $token = shift;
     my $hosthash = shift;
 
+    my $cref = $collector->collectorData( 'snmp' );
+    if( defined( $cref->{'snmpargs'}{$hosthash} ) )
+    {
+        return $cref->{'snmpargs'}{$hosthash};
+    }
+
     my ($ipaddr, $port, $community) = split(/\|/o, $hosthash);
 
     my $version = $collector->param($token, 'snmp-version');
-    my @ret = ( -hostname     => $ipaddr,
+    my $ret = [ -hostname     => $ipaddr,
                 -port         => $port,
                 -timeout      => $collector->param($token, 'snmp-timeout'),
                 -retries      => $collector->param($token, 'snmp-retries'),
-                -version      => $version );
-
+                -version      => $version ];
+    
     foreach my $arg ( qw(-localaddr -localport) )
     {
         if( defined( $collector->param($token, 'snmp' . $arg) ) )
         {
-            push( @ret, $arg, $collector->param($token, 'snmp' . $arg) );
+            push( @{$ret}, $arg, $collector->param($token, 'snmp' . $arg) );
         }
     }
             
     if( $version eq '1' or $version eq '2c' )
     {
-        push( @ret, '-community', $community );
+        push( @{$ret}, '-community', $community );
     }
     else
     {
-        push( @ret, -username, $community);
+        push( @{$ret}, -username, $community);
 
         foreach my $arg ( qw(-authkey -authpassword -authprotocol
                              -privkey -privpassword -privprotocol) )
         {
             if( defined( $collector->param($token, 'snmp' . $arg) ) )
             {
-                push( @ret, $arg, $collector->param($token, 'snmp' . $arg) );
+                push( @{$ret},
+                      $arg, $collector->param($token, 'snmp' . $arg) );
             }
         }
     }
 
-    return @ret;
+    $cref->{'snmpargs'}{$hosthash} = $ret;
+    return $ret;
 }
               
 
@@ -288,9 +324,10 @@ sub openBlockingSession
     my $collector = shift;
     my $token = shift;
     my $hosthash = shift;
-    
+
+    my $args = snmpSessionArgs( $collector, $token, $hosthash );
     my ($session, $error) =
-        Net::SNMP->session( snmpSessionArgs( $collector, $token, $hosthash ),
+        Net::SNMP->session( @{$args},
                             -nonblocking  => 0,
                             -translate    => ['-all', 0, '-octetstring', 1] );
     if( not defined($session) )
@@ -307,8 +344,10 @@ sub openNonblockingSession
     my $token = shift;
     my $hosthash = shift;
 
+    my $args = snmpSessionArgs( $collector, $token, $hosthash );
+    
     my ($session, $error) =
-        Net::SNMP->session( snmpSessionArgs( $collector, $token, $hosthash ),
+        Net::SNMP->session( @{$args},
                             -nonblocking  => 0x1,
                             -translate    => ['-timeticks' => 0] );
     if( not defined($session) )
@@ -466,10 +505,11 @@ sub lookupMap
     my $key = shift;
 
     my $cref = $collector->collectorData( 'snmp' );
+    my $maphash = join('#', $hosthash, $map);
 
     if( not defined( $maps{$hosthash}{$map} ) )
-    {
-        if( $mapLookupScheduled{$hosthash}{$map} )
+    {        
+        if( $mapLookupScheduled{$maphash} )
         {
             return undef;
         }
@@ -493,31 +533,51 @@ sub lookupMap
                              -callback => [\&mapLookupCallback,
                                            $collector, $hosthash, $map] );
 
-        $mapLookupScheduled{$hosthash}{$map} = 1;
+        $mapLookupScheduled{$maphash} = 1;
 
+        if( not $snmpV1Hosts{$hosthash} )
+        {
+            $mapsExpire{$maphash} =
+                int( time() + $mapsRefreshPeriod +
+                     rand( $mapsRefreshPeriod * $mapsRefreshRandom ) );
+        }
+        
         return undef;
     }
 
-    my $value = $maps{$hosthash}{$map}{$key};
-    if( not defined $value )
+    if( defined( $key ) )
     {
-        Error("Cannot find value $key in map $map for $hosthash in ".
-              $collector->path($token));
-        if( defined ( $maps{$hosthash}{$map} ) )
+        my $value = $maps{$hosthash}{$map}{$key};
+        if( not defined $value )
         {
-            Error("Current map follows");
-            while( my($key, $val) = each
-                   %{$maps{$hosthash}{$map}} )
+            Error("Cannot find value $key in map $map for $hosthash in ".
+                  $collector->path($token));
+            if( defined ( $maps{$hosthash}{$map} ) )
             {
-                Error("'$key' => '$val'");
+                Error("Current map follows");
+                while( my($key, $val) = each
+                       %{$maps{$hosthash}{$map}} )
+                {
+                    Error("'$key' => '$val'");
+                }
             }
+            return 'notfound';
+        }
+        else
+        {
+            if( not $snmpV1Hosts{$hosthash} )
+            {
+                $cref->{'mapsDependentTokens'}{$maphash}{$token} = 1;
+                $cref->{'mapsRelatedMaps'}{$token}{$maphash} = 1;
+            }
+            
+            return $value;
         }
-        return 'notfound';
     }
     else
     {
-        return $value;
-    }    
+        return undef;
+    }
 }
 
 
@@ -569,15 +629,15 @@ sub probablyDead
 
     # Stop all collection for this host, until next initTargetAttributes
     # is successful
-    delete $cref->{'reptoken'}{$hosthash};
+    delete $cref->{'activehosts'}{$hosthash};
 
     my $probablyAlive = 1;
 
-    if( defined( $cref->{'hostUnreachableSeen'}{$hosthash} ) )
+    if( defined( $hostUnreachableSeen{$hosthash} ) )
     {
         if( $Torrus::Collector::SNMP::unreachableTimeout > 0 and
             time() -
-            $cref->{'hostUnreachableSeen'}{$hosthash} >
+            $hostUnreachableSeen{$hosthash} >
             $Torrus::Collector::SNMP::unreachableTimeout )
         {
             $probablyAlive = 0;
@@ -585,7 +645,7 @@ sub probablyDead
     }
     else
     {
-        $cref->{'hostUnreachableSeen'}{$hosthash} = time();
+        $hostUnreachableSeen{$hosthash} = time();
     }
 
     if( $probablyAlive )
@@ -613,11 +673,10 @@ sub probablyDead
         {
             $collector->deleteTarget($token);
         }
-        
-        delete $cref->{'reptoken'}{$hosthash};
-        delete $cref->{'hostUnreachableSeen'}{$hosthash};
-        delete $cref->{'hostUnreachableRetry'}{$hosthash};
-        $cref->{'unreachableHostDeleted'}{$hosthash} = 1;
+                
+        delete $hostUnreachableSeen{$hosthash};
+        delete $hostUnreachableRetry{$hosthash};
+        $unreachableHostDeleted{$hosthash} = 1;
     }
     
     return $probablyAlive;
@@ -633,13 +692,13 @@ sub checkUnreachableRetry
     my $cref = $collector->collectorData( 'snmp' );
 
     my $ret = 1;
-    if( $cref->{'hostUnreachableSeen'}{$hosthash} )
+    if( $hostUnreachableSeen{$hosthash} )
     {
-        my $lastRetry = $cref->{'hostUnreachableRetry'}{$hosthash};
+        my $lastRetry = $hostUnreachableRetry{$hosthash};
 
         if( not defined( $lastRetry ) )
         {
-            $lastRetry = $cref->{'hostUnreachableSeen'}{$hosthash};
+            $lastRetry = $hostUnreachableSeen{$hosthash};
         }
             
         if( time() < $lastRetry +
@@ -649,7 +708,7 @@ sub checkUnreachableRetry
         }
         else
         {
-            $cref->{'hostUnreachableRetry'}{$hosthash} = time();
+            $hostUnreachableRetry{$hosthash} = time();
         }            
     }
     
@@ -663,7 +722,7 @@ sub isHostDead
     my $hosthash = shift;
 
     my $cref = $collector->collectorData( 'snmp' );
-    return $cref->{'unreachableHostDeleted'}{$hosthash};
+    return $unreachableHostDeleted{$hosthash};
 }
 
 
@@ -673,9 +732,9 @@ sub hostReachableAgain
     my $hosthash = shift;
     
     my $cref = $collector->collectorData( 'snmp' );
-    if( exists( $cref->{'hostUnreachableSeen'}{$hosthash} ) )
+    if( exists( $hostUnreachableSeen{$hosthash} ) )
     {
-        delete $cref->{'hostUnreachableSeen'}{$hosthash};
+        delete $hostUnreachableSeen{$hosthash};
     }
 }
 
@@ -705,6 +764,12 @@ sub deleteTarget
     }
 
     delete $cref->{'needsRemapping'}{$token};
+    
+    foreach my $maphash ( keys %{$cref->{'mapsRelatedMaps'}{$token}} )
+    {
+        delete $cref->{'mapsDependentTokens'}{$maphash}{$token};
+    }
+    delete $cref->{'mapsRelatedMaps'}{$token};
 }
 
 # Main collector cycle
@@ -722,11 +787,27 @@ sub runCollector
     # within one address
 
     my @sessions;
-    
-    while( my ($hosthash, $token) = each %{$cref->{'reptoken'}} )
+
+    foreach my $hosthash ( keys %{$cref->{'activehosts'}} )
     {
+        my @oids = sort keys %{$cref->{'targets'}{$hosthash}};
+        
+        # Find one representative token for the host
+        
+        if( scalar( @oids ) == 0 )
+        {
+            next;
+        }
+        
+        my @reptokens = keys %{$cref->{'targets'}{$hosthash}{$oids[0]}};
+        if( scalar( @reptokens ) == 0 )
+        {
+            next;
+        }
+        my $reptoken = $reptokens[0];
+                    
         my $session =
-            openNonblockingSession( $collector, $token, $hosthash );
+            openNonblockingSession( $collector, $reptoken, $hosthash );
         
         if( not defined($session) )
         {
@@ -740,7 +821,6 @@ sub runCollector
         
         my $oids_per_pdu = $cref->{'oids_per_pdu'}{$hosthash};
 
-        my @oids = sort keys %{$cref->{'targets'}{$hosthash}};
         my @pdu_oids = ();
         my $delay = 0;
         
@@ -772,10 +852,13 @@ sub runCollector
                 my $pdu_tokens = {};
                 foreach my $oid ( @pdu_oids )
                 {
-                    foreach my $token
-                        ( keys %{$cref->{'targets'}{$hosthash}{$oid}} )
+                    if( defined( $cref->{'targets'}{$hosthash}{$oid} ) )
                     {
-                        $pdu_tokens->{$oid}{$token} = 1;
+                        foreach my $token
+                            ( keys %{$cref->{'targets'}{$hosthash}{$oid}} )
+                        {
+                            $pdu_tokens->{$oid}{$token} = 1;
+                        }
                     }
                 }
                 my $result =
@@ -899,6 +982,11 @@ sub callback
                 $value eq 'endOfMibView' )
             {
                 Error("Error retrieving $oid from $hosthash: $value");
+                
+                foreach my $token ( keys %{$pdu_tokens->{$oid}} )
+                {
+                    $collector->deleteTarget($token);
+                }
             }
             else
             {
@@ -928,6 +1016,83 @@ sub postProcess
     my $collector = shift;
     my $cref = shift;
 
+    # It could happen that postProcess is called for a collector which
+    # has no targets, and therefore it's the only place where we can
+    # initialize these variables
+    
+    if( not defined( $cref->{'mapsLastExpireChecked'} ) )
+    {
+        $cref->{'mapsLastExpireChecked'} = 0;
+    }
+
+    if( not defined( $cref->{'mapsRefreshed'} ) )
+    {
+        $cref->{'mapsRefreshed'} = [];
+    }
+    
+    # look if some maps are ready after last expiration check
+    if( scalar( @{$cref->{'mapsRefreshed'}} ) > 0 )
+    {
+        foreach my $maphash ( @{$cref->{'mapsRefreshed'}} )
+        {
+            foreach my $token
+                ( keys %{$cref->{'mapsDependentTokens'}{$maphash}} )
+            {
+                $cref->{'needsRemapping'}{$token} = 1;
+            }
+        }
+        $cref->{'mapsRefreshed'} = [];
+    }
+
+    my $now = time();
+    
+    if( $cref->{'mapsLastExpireChecked'} + $mapsExpireCheckPeriod <= $now )
+    {
+        $cref->{'mapsLastExpireChecked'} = $now;
+        
+        # Check the maps expiration and arrange lookup for expired
+        
+        while( my ( $maphash, $expire ) = each %mapsExpire )
+        {
+            if( $expire <= $now and not $mapLookupScheduled{$maphash} )
+            {                
+                my ( $hosthash, $map ) = split( /\#/o, $maphash );
+
+                if( $unreachableHostDeleted{$hosthash} )
+                {
+                    # This host is no longer polled. Remove the leftovers
+                    
+                    delete $mapsExpire{$maphash};
+                    delete $maps{$hosthash};
+                }
+                else
+                {
+                    # Find one representative token for the map
+                    my @tokens =
+                        keys %{$cref->{'mapsDependentTokens'}{$maphash}};
+                    if( scalar( @tokens ) == 0 )
+                    {
+                        next;
+                    }
+                    my $reptoken = $tokens[0];
+
+                    delete $maps{$hosthash}{$map};
+
+                    # this will schedule the map retrieval for the next
+                    # collector cycle
+                    Debug('Refreshing map: ' . $maphash);
+                
+                    lookupMap( $collector, $reptoken,
+                               $hosthash, $map, undef );
+
+                    # After the next collector period, the maps will be
+                    # ready and tokens may be updated without losing the data
+                    push( @{$cref->{'mapsRefreshed'}}, $maphash );
+                }
+            }                
+        }
+    }
+    
     foreach my $token ( keys %{$cref->{'needsRemapping'}} )
     {
         delete $cref->{'needsRemapping'}{$token};
