@@ -1,4 +1,4 @@
-#  Copyright (C) 2002  Stanislav Sinyagin
+#  Copyright (C) 2002-2007  Stanislav Sinyagin
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -14,7 +14,7 @@
 #  along with this program; if not, write to the Free Software
 #  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 
-# $Id: Writer.pm,v 1.11 2006/12/03 12:46:09 ssinyagin Exp $
+# $Id: Writer.pm,v 1.15 2007/06/21 08:00:19 ssinyagin Exp $
 # Stanislav Sinyagin <ssinyagin@yahoo.com>
 
 #
@@ -34,16 +34,6 @@ use Torrus::ServiceID;
 use strict;
 use Digest::MD5 qw(md5); # needed as hash function
 
-# Global list of parameters requiring space removal
-my @remspace =
-    (qw(storage-type monitor action launch-when rpn-expr tokenset-member
-        setenv-params setenv-dataexpr ds-names value-map rrgraph-views
-        print-cf hrules display-rpn-expr));
-
-foreach my $param ( @remspace )
-{
-    $Torrus::ConfigTree::Writer::remove_space{$param} = 1;
-}
 
 our %multigraph_remove_space =
     ('ds-expr-' => 1,
@@ -72,6 +62,9 @@ sub new
     $self->{'mayRunCollector'} =
         Torrus::SiteConfig::mayRunCollector( $self->treeName() );
 
+    $self->{'collectorInstances'} =
+        Torrus::SiteConfig::collectorInstances( $self->treeName() );
+    
     return $self;
 }
 
@@ -93,7 +86,7 @@ sub setParam
     my $param = shift;
     my $value = shift;
 
-    if( $Torrus::ConfigTree::Writer::remove_space{$param} )
+    if( $self->getParamProperty( $param, 'remspace' ) )
     {
         $value =~ s/\s+//go;
     }
@@ -110,7 +103,7 @@ sub setNodeParam
     my $param = shift;
     my $value = shift;
 
-    if( $Torrus::ConfigTree::Writer::remove_space{$param} )
+    if( $self->getParamProperty( $param, 'remspace' ) )
     {
         $value =~ s/\s+//go;
     }
@@ -121,6 +114,18 @@ sub setNodeParam
 }
 
 
+sub setParamProperty
+{
+    my $self = shift;
+    my $param = shift;
+    my $prop = shift;
+    my $value = shift;
+
+    $self->{'paramprop'}{$prop}{$param} = $value;
+    $self->{'db_paramprops'}->put( $param . ':' . $prop, $value );
+}
+
+
 sub initRoot
 {
     my $self  = shift;
@@ -440,6 +445,37 @@ sub postProcessNodes
             }
             elsif( $dsType eq 'collector' and $self->{'mayRunCollector'} )
             {
+                # Split the collecting job between collector instances
+                my $instance = 0;
+                my $nInstances = $self->{'collectorInstances'};
+
+                my $oldOffset =
+                    $self->getNodeParam($token, 'collector-timeoffset');
+                my $newOffset = $oldOffset;
+                
+                my $period =
+                    $self->getNodeParam($token, 'collector-period');
+                
+                if( $nInstances > 1 )
+                {
+                    my $hashString =
+                        $self->getNodeParam($token,
+                                            'collector-instance-hashstring');
+                    if( not defined( $hashString ) )
+                    {
+                        Error('collector-instance-hashstring is not defined ' .
+                              'in ' . $self->path( $token ));
+                        $hashString = '';
+                    }
+                    
+                    $instance =
+                        unpack( 'N', md5( $hashString ) ) % $nInstances;
+                }          
+
+                $self->setNodeParam( $token,
+                                     'collector-instance',
+                                     $instance );
+                
                 my $dispersed =
                     $self->getNodeParam($token,
                                         'collector-dispersed-timeoffset');
@@ -482,20 +518,31 @@ sub postProcessNodes
                             my $step = $p{'collector-timeoffset-step'};
                             my $hashString =
                                 $p{'collector-timeoffset-hashstring'};
-
-                            my $bucketSize = int( ($max - $min) / $step );
-                            my $offset = $min +
-                                $step * ( unpack( 'N', md5( $hashString ) ) %
-                                          $bucketSize );
                             
-                            Debug('Hashed offset ' . $offset . ' for ' .
-                                  $token);
-                            $self->setNodeParam( $token,
-                                                 'collector-timeoffset',
-                                                 $offset );
+                            my $bucketSize = int( ($max - $min) / $step );
+                            $newOffset =
+                                $min
+                                +
+                                $step * ( unpack( 'N', md5( $hashString ) ) %
+                                          $bucketSize )
+                                +
+                                $instance * int( $step / $nInstances );
                         }
                     }
                 }
+                else
+                {
+                    $newOffset += $instance * int( $period / $nInstances ); 
+                } 
+
+                $newOffset %= $period;
+                
+                if( $newOffset != $oldOffset )
+                {
+                    $self->setNodeParam( $token,
+                                         'collector-timeoffset',
+                                         $newOffset );
+                }
 
                 my $storagetypes =
                     $self->getNodeParam( $token, 'storage-type' );
