@@ -14,7 +14,7 @@
 #  along with this program; if not, write to the Free Software
 #  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 
-# $Id: XMLCompiler.pm,v 1.4 2006/12/03 12:46:09 ssinyagin Exp $
+# $Id: XMLCompiler.pm,v 1.6 2007/04/10 14:28:34 ssinyagin Exp $
 # Stanislav Sinyagin <ssinyagin@yahoo.com>
 
 
@@ -114,6 +114,11 @@ sub compile
         }
     }
 
+    foreach $node ( $root->getElementsByTagName('param-properties') )
+    {
+        $ok = $self->compile_paramprops( $node ) ? $ok:0;
+    }
+
     if( not $self->{'-NoDSRebuild'} )
     {
         foreach $node ( $root->getElementsByTagName('definitions') )
@@ -177,6 +182,31 @@ sub compile_definitions
 }
 
 
+sub compile_paramprops
+{
+    my $self = shift;
+    my $node = shift;
+    my $ok = 1;
+
+    foreach my $def ( $node->getChildrenByTagName('prop') )
+    {
+        my $param = $def->getAttribute('param'); 
+        my $prop = $def->getAttribute('prop');
+        my $value = $def->getAttribute('value');
+        if( not $param or not $prop or not defined($value) )
+        {
+            Error("Property definition error"); $ok = 0;
+        }
+        else
+        {
+            $self->setParamProperty($param, $prop, $value);
+        }
+    }
+    return $ok;
+}
+
+
+
 # Process <param name="name" value="value"/> and put them into DB.
 # Usage: $self->compile_params($node, $name);
 
@@ -332,121 +362,6 @@ sub compile_subtrees
         $ok = $self->setAlias($token, $apath) ? $ok:0;
     }
 
-    # Handle file patterns -- we're still in compile_subtrees()
-
-    foreach my $fp ( $node->getChildrenByTagName('filepattern') )
-    {
-        my $type = $fp->getAttribute('type');
-        my $name = $fp->getAttribute('name');
-        my $file_re = $fp->getAttribute('filere');
-        my $dirname = $self->getNodeParam($token, 'data-dir');
-
-        if($type ne 'subtree' and $type ne 'leaf')
-        {
-            Error("Unknown filepattern type: $type at $path"); $ok = 0;
-        }
-        elsif( not defined($name) or not defined($file_re) )
-        {
-            Error("Filepattern name or RE not defined at $path"); $ok = 0;
-        }
-        elsif( not defined($dirname) )
-        {
-            Error("data-dir parameter not defined at $path"); $ok = 0;
-        }
-        elsif( not -d $dirname )
-        {
-            Error("Directory $dirname does not exist at $path"); $ok = 0;
-        }
-        else
-        {
-            $file_re = $self->expand_name($path, $file_re);
-
-            # Read the directory and match the pattern
-            my %applied = ();
-            opendir(DIR, $dirname) or die "can't opendir $dirname: $!";
-            while( (my $fname = readdir(DIR)) )
-            {
-                if( $fname =~ /$file_re/ )
-                {
-                    my $newnodename = eval($name);
-                    if( defined $applied{$newnodename} )
-                    {
-                        Error("Filepattern gives non-unique names: " .
-                              "name=\"$name\" filere=\"$file_re\" at $path");
-                        $ok = 0;
-                    }
-                    else
-                    {
-                        $applied{$newnodename} = $fname;
-                    }
-                }
-            }
-            closedir DIR;
-            # Clone the contents of filepattern into the main tree
-            my %detailedmatched = ();
-            foreach my $newnodename (keys %applied)
-            {
-                my $newnode = XML::LibXML::Element->new( $type );
-                $newnode->setAttribute('name', $newnodename);
-                $node->parentNode()->appendChild($newnode);
-
-                my $childname = $newnodename;
-                $childname .= '/' if $type eq 'subtree';
-                my $newnodetoken = $self->addChild($token, $childname);
-                my $newnodepath = $path.$childname;
-
-                $self->setNodeParam($newnodetoken, 'data-file',
-                                $applied{$newnodename});
-
-                foreach my $fpchild ($fp->childNodes())
-                {
-                    if( $fpchild->nodeName() eq 'detailed' )
-                    {
-                        my $match =  $fpchild->getAttribute('match');
-                        if( not defined $match )
-                        {
-                            Error("Detailed should have match at $path");
-                            $ok = 0;
-                        }
-                        else
-                        {
-                            if( not defined $detailedmatched{$match} )
-                            {
-                                $detailedmatched{$match} = 0;
-                            }
-                            if( $match eq $newnodename )
-                            {
-                                $detailedmatched{$match} = 1;
-                                foreach my $detnode ( $fpchild->childNodes() )
-                                {
-                                    $newnode->
-                                        appendChild($detnode->cloneNode(1));
-                                }
-                            }
-                        }
-                    }
-                    else
-                    {
-                        $newnode->appendChild($fpchild->cloneNode(1));
-                    }
-                }
-                $ok = $self->
-                    compile_subtrees( $newnode, $newnodetoken ) ? $ok:0;
-            }
-
-            # Check if any of detailed have not matched
-
-            foreach my $match (keys %detailedmatched)
-            {
-                if( not $detailedmatched{$match} )
-                {
-                    Warn("Detailed match \"$match\" have not matched any ".
-                         "file at $path");
-                }
-            }
-        }
-    }
-
     foreach my $setvar ( $node->getChildrenByTagName('setvar') )        
     {
         my $name = $setvar->getAttribute('name');
