@@ -14,7 +14,7 @@
 #  along with this program; if not, write to the Free Software
 #  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 
-# $Id: DevDiscover.pm,v 1.24 2006/12/14 08:02:26 ssinyagin Exp $
+# $Id: DevDiscover.pm,v 1.29 2007/05/04 11:08:04 ssinyagin Exp $
 # Stanislav Sinyagin <ssinyagin@yahoo.com>
 
 # Core SNMP device discovery module
@@ -39,6 +39,12 @@ BEGIN
     }
 }
 
+# Custom overlays for templates
+# overlayName ->
+#     'Module::templateName' -> { 'name' => 'templateName',
+#                                 'source' => 'filename.xml' }
+our %templateOverlays;
+
 our @requiredParams =
     (
      'snmp-port',
@@ -55,6 +61,7 @@ $defaultParams{'rrd-hwpredict'} = 'no';
 $defaultParams{'domain-name'} = '';
 $defaultParams{'host-subtree'} = '';
 $defaultParams{'snmp-check-sysuptime'} = 'yes';
+$defaultParams{'show-recursive'} = 'yes';
 
 our @copyParams =
     ( 'collector-period',
@@ -67,6 +74,7 @@ our @copyParams =
       'domain-name',
       'monitor-period',
       'monitor-timeoffset',
+      'show-recursive',
       'snmp-host',
       'snmp-port',
       'snmp-localaddr',
@@ -124,12 +132,19 @@ sub new
     }
 
     $self->{'datadirs'} = {};
+    $self->{'globalData'} = {};
 
     return $self;
 }
 
 
 
+sub globalData
+{
+    my $self = shift;
+    return $self->{'globalData'};
+}
+
 
 sub discover
 {
@@ -445,6 +460,31 @@ sub buildConfig
 
             my $data = $devdetails->data();
 
+            my @registryOverlays = ();
+            if( defined( $devdetails->param('template-registry-overlays' ) ) )
+            {
+                my @overlayNames = 
+                    split(/\s*,\s*/,
+                          $devdetails->param('template-registry-overlays' ));
+                foreach my $overlayName ( @overlayNames )
+                {
+                    if( defined( $templateOverlays{$overlayName}) )
+                    {
+                        push( @registryOverlays,
+                              $templateOverlays{$overlayName} );
+                    }
+                    else
+                    {
+                        Error('Cannot find the template overlay named ' .
+                              $overlayName);
+                    }
+                }
+            }
+
+            # we should call this anyway, in order to flush the overlays
+            # set by previous host
+            $cb->setRegistryOverlays( @registryOverlays );            
+            
             if( $devdetails->param('disable-snmpcollector' ) eq 'yes' )
             {
                 push( @{$data->{'templates'}}, '::viewonly-defaults' );
@@ -490,13 +530,24 @@ sub buildConfig
                   $devdetails->getDevTypes() )
             {
                 &{$reg->{$devtype}{'buildConfig'}}
-                ( $devdetails, $cb, $devNode );
+                ( $devdetails, $cb, $devNode, $self->{'globalData'} );
             }
 
             $cb->{'statistics'}{'hosts'}++;
         }
     }
 
+    foreach my $devtype
+        ( sort {$reg->{$a}{'sequence'} <=> $reg->{$b}{'sequence'}}
+          keys %{$reg} )
+    {
+        if( defined( $reg->{$devtype}{'buildGlobalConfig'} ) )
+        {
+            &{$reg->{$devtype}{'buildGlobalConfig'}}($cb,
+                                                     $self->{'globalData'});
+        }
+    }
+    
     if( defined( $self->{'define-tokensets'} ) )
     {
         my $tsetsNode = $cb->startTokensets();
