@@ -14,7 +14,7 @@
 #  along with this program; if not, write to the Free Software
 #  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 
-# $Id: CiscoIOS_Docsis.pm,v 1.15 2006/04/12 12:21:10 ssinyagin Exp $
+# $Id: CiscoIOS_Docsis.pm,v 1.16 2006/09/27 08:56:30 ssinyagin Exp $
 # Stanislav Sinyagin <ssinyagin@yahoo.com>
 
 # DOCSIS interface, Cisco specific
@@ -237,7 +237,7 @@ sub buildConfig
 
         # Override the overview shortcus defined in rfc2670.docsis-if.xml
         
-        my $shortcuts = 'snr,fec,modems,util';
+        my $shortcuts = 'snr,fec,freq,modems,util';
         if( $devdetails->hasCap('cdxIfUpChannelMaxUGSLastFiveMins') )
         {
             $shortcuts .= ',ugs';
