@@ -14,7 +14,7 @@
 #  along with this program; if not, write to the Free Software
 #  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 
-# $Id: CiscoIOS_MacAccounting.pm,v 1.7 2007/01/05 20:48:15 ssinyagin Exp $
+# $Id: CiscoIOS_MacAccounting.pm,v 1.9 2007/04/12 07:55:52 ssinyagin Exp $
 # Stanislav Sinyagin <ssinyagin@yahoo.com>
 
 # Cisco IOS MAC accounting
@@ -144,6 +144,13 @@ sub discover
         next if ( $phyAddr eq '0xffffffffffff' );
         
         my $peerIP = $interface->{'mediaToIpNet'}{$phyAddr};
+        if( not defined( $peerIP ) )
+        {
+            # Try in the global table, as the ARP is stored per subinterface,
+            # and MAC accounting is on main interface
+            $peerIP = $data->{'mediaToIpNet'}{$phyAddr};
+        }
+        
         if( not defined( $peerIP ) )
         {
             # high logging level, because who cares about staled entries?
@@ -283,7 +290,8 @@ sub buildConfig
             'interface-nick'       => $peer->{'ifNick'},
             'comment'              => $peer->{'description'},
             'descriptive-nickname' => $peer->{'subtreeName'},
-            'precedence'           => 65000 - $peer->{'peerAS'}
+            'precedence'           => 65000 - $peer->{'peerAS'},
+            'searchable'           => 'yes'
             };
 
         my $peerNode = $cb->addSubtree
