@@ -14,7 +14,7 @@
 #  along with this program; if not, write to the Free Software
 #  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 
-# $Id: Monitor.pm,v 1.8 2005/06/16 13:06:46 ssinyagin Exp $
+# $Id: Monitor.pm,v 1.16 2006/02/27 07:52:08 ssinyagin Exp $
 # Stanislav Sinyagin <ssinyagin@yahoo.com>
 
 package Torrus::Monitor;
@@ -146,9 +146,38 @@ sub check_expression
 
     my ($value, $timestamp) = $obj->{'da'}->read( $config_tree, $token );
     $value = 'UNKN' unless defined($value);
-    $obj->{'value'} = $value;
     
     my $expr = $value . ',' . $config_tree->getParam($mname,'rpn-expr');
+    $expr = $self->substitute_vars( $config_tree, $obj, $expr );
+
+    my $display_expr = $config_tree->getParam($mname,'display-rpn-expr');
+    if( defined( $display_expr ) )
+    {
+        $display_expr =
+            $self->substitute_vars( $config_tree, $obj,
+                                    $value . ',' . $display_expr );
+        my ($dv, $dt) = $obj->{'da'}->read_RPN( $config_tree, $token,
+                                                $display_expr, $timestamp );
+        $obj->{'display_value'} = $dv;
+    }
+    else
+    {
+        $obj->{'display_value'} = $value;
+    }
+    
+    return $obj->{'da'}->read_RPN( $config_tree, $token, $expr, $timestamp );
+}
+
+
+sub substitute_vars
+{
+    my $self = shift;
+    my $config_tree = shift;
+    my $obj = shift;
+    my $expr = shift;
+    
+    my $token = $obj->{'token'};
+    my $mname = $obj->{'mname'};
 
     if( index( $expr, '#' ) >= 0 )
     {
@@ -194,8 +223,10 @@ sub check_expression
         }
 
     }
-    return $obj->{'da'}->read_RPN( $config_tree, $token, $expr, $timestamp );
+
+    return $expr;
 }
+    
 
 
 sub setAlarm
@@ -258,15 +289,17 @@ sub setAlarm
         Debug("Event: $event, Monitor: $mname, Token: $token");
         $obj->{'event'} = $event;
         
+        my $action_token = $token;
+        
         my $action_target =
             $config_tree->getNodeParam($token, 'monitor-action-target');
         if( defined( $action_target ) )
         {
             Debug('Action target redirected to ' . $action_target);
-            $token = $config_tree->getRelative($token, $action_target);
-            $obj->{'token'} = $token;
-            Debug('Redirected to token ' . $token);
+            $action_token = $config_tree->getRelative($token, $action_target);
+            Debug('Redirected to token ' . $action_token);
         }
+        $obj->{'action_token'} = $action_token;
 
         foreach my $aname (split(',',
                                  $config_tree->getParam($mname, 'action')))
@@ -296,7 +329,7 @@ sub run_event_tset
     my $aname = shift;
     my $obj = shift;
 
-    my $token = $obj->{'token'};
+    my $token = $obj->{'action_token'};
     my $event = $obj->{'event'};
     
     if( $event eq 'set' or $event eq 'forget' )
@@ -322,7 +355,7 @@ sub run_event_exec
     my $aname = shift;
     my $obj = shift;
 
-    my $token = $obj->{'token'};
+    my $token = $obj->{'action_token'};
     my $event = $obj->{'event'};
     my $mname = $obj->{'mname'};
     my $timestamp = $obj->{'timestamp'};
@@ -364,9 +397,24 @@ sub run_event_exec
         $ENV{'TORRUS_MCOMMENT'}  = $config_tree->getParam($mname, 'comment');
         $ENV{'TORRUS_TSTAMP'}    = $timestamp;
 
-        if( defined( $obj->{'value'} ) )
+        if( defined( $obj->{'display_value'} ) )
         {
-            $ENV{'TORRUS_VALUE'} = $obj->{'value'};
+            $ENV{'TORRUS_VALUE'} = $obj->{'display_value'};
+
+            my $format = $config_tree->getParam($mname, 'display-format');
+            if( not defined( $format ) )
+            {
+                $format = '%.2f';
+            }
+
+            $ENV{'TORRUS_DISPLAY_VALUE'} =
+                sprintf( $format, $obj->{'display_value'} );
+        }
+
+        my $severity = $config_tree->getParam($mname, 'severity');
+        if( defined( $severity ) )
+        {
+            $ENV{'TORRUS_SEVERITY'} = $severity;
         }
         
         my $setenv_params =
@@ -376,12 +424,15 @@ sub run_event_exec
         {
             foreach my $param ( split( ',', $setenv_params ) )
             {
-                my $value = $config_tree->getNodeParam( $token, $param );
+                # We retrieve the param from the monitored token, not
+                # from action-token
+                my $value = $config_tree->getNodeParam( $obj->{'token'},
+                                                        $param );
                 if( not defined $value )
                 {
                     Warn('Parameter ' . $param . ' referenced in action '.
                          $aname . ', but not defined for ' .
-                         $ENV{'TORRUS_NODEPATH'});
+                         $config_tree->path($obj->{'token'}));
                     $value = '';
                 }
                 $param =~ s/\W/_/g;
@@ -486,7 +537,7 @@ sub beforeRun
 
         # Remove the alarms state information
         $self->{'db_alarms'} = new Torrus::DB('monitor_alarms',
-                                              -Subdir => $self->{'tree_name'},
+                                              -Subdir => $tree,
                                               -WriteAccess => 1);
         $self->{'db_alarms'}->trunc();
         
