@@ -14,7 +14,7 @@
 #  along with this program; if not, write to the Free Software
 #  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 
-# $Id: HTML.pm,v 1.10 2006/03/01 11:04:37 ssinyagin Exp $
+# $Id: HTML.pm,v 1.13 2007/04/18 16:05:20 ssinyagin Exp $
 # Stanislav Sinyagin <ssinyagin@yahoo.com>
 
 package Torrus::Renderer::HTML;
@@ -22,6 +22,7 @@ package Torrus::Renderer::HTML;
 use strict;
 
 use Torrus::ConfigTree;
+use Torrus::Search;
 use Torrus::Log;
 
 use URI::Escape;
@@ -106,7 +107,9 @@ sub render_html
         'timestamp'  => sub { return time2str($Torrus::Renderer::timeFormat,
                                               time()); },
         'verifyDate'  => sub { return verifyDate($_[0]); },
-        'markup'     => sub{ return $self->translateMarkup( @_ ); }
+        'markup'     => sub{ return $self->translateMarkup( @_ ); },
+        'searchEnabled' => $Torrus::Renderer::searchEnabled,
+        'searchResults' => sub { return $self->doSearch($config_tree, $_[0]); }
     };
     
     
@@ -279,7 +282,7 @@ sub rrPrint
     my $view = shift;
 
     my @ret = ();
-    my($fname, $mimetype) =  $self->render($config_tree, $token, $view);
+    my($fname, $mimetype) = $self->render( $config_tree, $token, $view );
 
     if( $mimetype ne 'text/plain' )
     {
@@ -461,6 +464,27 @@ sub reportsUrl
 }
 
 
+sub doSearch
+{
+    my $self = shift;
+    my $config_tree = shift;
+    my $string = shift;
+    
+
+    my $tree = $config_tree->treeName();
+    
+    my $sr = new Torrus::Search;
+    $sr->openTree( $tree );
+    my $result = $sr->searchPrefix( $string, $tree );
+    $sr->closeTree( $tree );
+
+    my $ret = [];
+    push( @{$ret}, sort {$a->[0] cmp $b->[0]} @{$result} );
+    
+    return $ret;
+}
+
+
 1;
 
 
