@@ -14,7 +14,7 @@
 #  along with this program; if not, write to the Free Software
 #  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 
-# $Id: RRDtool.pm,v 1.7 2006/08/11 17:17:01 ssinyagin Exp $
+# $Id: RRDtool.pm,v 1.8 2007/03/23 10:44:53 ssinyagin Exp $
 # Stanislav Sinyagin <ssinyagin@yahoo.com>
 
 package Torrus::Renderer::RRDtool;
@@ -450,12 +450,22 @@ sub rrd_make_graphline
     {
         $legend = '';
     }
-    
-    my $linestyle =
-        $self->mkline( $config_tree->getParam($view, 'line-style') );
 
-    my $linecolor =
-        $self->mkcolor( $config_tree->getParam($view, 'line-color') );
+    my $styleval = $config_tree->getNodeParam($token, 'line-style');
+    if( not defined( $styleval ) or length( $styleval ) == 0 )
+    {
+        $styleval = $config_tree->getParam($view, 'line-style');
+    }
+    
+    my $linestyle = $self->mkline( $styleval );
+
+    my $colorval = $config_tree->getNodeParam($token, 'line-color');
+    if( not defined( $colorval ) or length( $colorval ) == 0 )
+    {
+        $colorval = $config_tree->getParam($view, 'line-color');
+    }
+    
+    my $linecolor = $self->mkcolor( $colorval );
 
     if( $self->rrd_if_gprint( $config_tree, $token ) )
     {
