@@ -14,7 +14,7 @@
 #  along with this program; if not, write to the Free Software
 #  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 
-# $Id: MonthlySrvUsage.pm,v 1.5 2005/10/24 16:22:49 ssinyagin Exp $
+# $Id: MonthlySrvUsage.pm,v 1.8 2007/07/26 09:03:38 ssinyagin Exp $
 # Stanislav Sinyagin <ssinyagin@yahoo.com>
 
 # For all service IDs available, build monthly usage figures:
@@ -88,7 +88,12 @@ sub generate
             my $rowtime = str2time( $row->{'srv_date'} . 'T' .
                                     $row->{'srv_time'} );
             my $pos = floor( ($rowtime - $self->{'StartUnixTime'}) / $step );
-            my $value = $row->{'value'};
+            my $value = Math::BigFloat->new( $row->{'value'} );
+            if( $value->is_nan() )
+            {
+                $value->bzero();
+                $row->{'value'} = 0;
+            }
             
             if( ( not defined( $aligned[$pos] ) ) or
                 $aligned[$pos] < $value )
@@ -103,7 +108,7 @@ sub generate
 
         # Set undefined values to zero and calculate the average
 
-        my $sum = 0;
+        my $sum = Math::BigFloat->new(0);
         my $unavailCount = 0;
         foreach my $pos ( 0 .. $#aligned )
         {
@@ -129,7 +134,8 @@ sub generate
         # Calculate the total volume, if it's a counter
         my $volume = Math::BigFloat->new(0);
         my $volumeDefined = 0;
-        if( $params->{'dstype'} =~ /^COUNTER/ )
+        if( not defined( $params->{'dstype'} ) or
+            $params->{'dstype'} =~ /^COUNTER/ )
         {
             $volumeDefined = 1;
             foreach my $row ( @{$data} )
@@ -142,7 +148,8 @@ sub generate
 
         my $usageUnits = '';
         my $volumeUnits = '';
-        if( $params->{'units'} eq 'bytes' )
+        if( not defined( $params->{'units'} ) or
+            $params->{'units'} eq 'bytes' )
         {
             # Adjust bytes into megabit per second
             $usageUnits = 'Mbps';
