@@ -14,7 +14,7 @@
 #  along with this program; if not, write to the Free Software
 #  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 
-# $Id: SiteConfig.pm,v 1.5 2005/10/24 16:20:47 ssinyagin Exp $
+# $Id: SiteConfig.pm,v 1.6 2007/06/16 16:54:08 ssinyagin Exp $
 # Stanislav Sinyagin <ssinyagin@yahoo.com>
 
 ## %Torrus::Global::treeConfig manipulation
@@ -279,9 +279,16 @@ sub listTreeNames
 sub mayRunCollector
 {
     my $tree = shift;
-    return $Torrus::Global::treeConfig{$tree}{'run'}{'collector'};
+    my $run = $Torrus::Global::treeConfig{$tree}{'run'}{'collector'};
+    return( defined($run) and $run > 0 );   
 }
 
+sub collectorInstances
+{
+    my $tree = shift;
+    my $run = $Torrus::Global::treeConfig{$tree}{'run'}{'collector'};
+    return( (defined($run) and $run > 1) ? int($run) : 1 ); 
+}
 
 sub mayRunMonitor
 {
